@@ -206,7 +206,9 @@ impl MtcpConvergenceLayer {
                             info!("Received bundle: {} from {}", bndl.id(), peer_addr);
                             {
                                 tokio::spawn(async move {
-                                    crate::core::processing::receive(bndl.into()).await;
+                                    if let Err(err) = crate::core::processing::receive(bndl).await {
+                                        error!("Failed to process bundle: {}", err);
+                                    }
                                 });
                             }
                         } else {
