@@ -2,52 +2,61 @@ use super::BundleStore;
 use crate::core::bundlepack::{BundlePack, Constraint};
 use crate::CONFIG;
 use anyhow::{bail, Result};
-use log::error;
+use bp7::Bundle;
+use log::{debug, error};
+use std::convert::{TryFrom, TryInto};
 use std::fmt::Debug;
 
 #[derive(Debug, Clone)]
 pub struct SledBundleStore {
     bundles: sled::Tree,
+    metadata: sled::Tree,
     //bundles: HashMap<String, BundlePack>,
 }
 
 impl BundleStore for SledBundleStore {
-    fn push(&mut self, bp: &BundlePack) -> Result<()> {
+    fn push(&mut self, bndl: &Bundle) -> Result<()> {
         // TODO: check for duplicates, update, remove etc
-        if self.bundles.contains_key(bp.id())? {
-            bail!("Bundle already in store!");
+        if self.bundles.contains_key(bndl.id())? {
+            debug!("Bundle {} already in store, updating it!", bndl.id());
+        } else {
+            let bp = BundlePack::from(bndl);
+            if self.metadata.contains_key(bp.id())? {
+                bail!("Bundle metadata already in store!");
+            }
+            self.metadata.insert(bp.id().to_string(), bp.to_cbor())?;
+            self.metadata.flush()?;
         }
-        self.bundles.insert(bp.id().to_string(), bp.to_cbor())?;
+        // TODO: eliminate double clone (here and in BundlePack::from)
+        self.bundles
+            .insert(bndl.id().to_string(), bndl.clone().to_cbor())?;
         self.bundles.flush()?;
+
         Ok(())
     }
-    fn update(&mut self, bp: &BundlePack) -> Result<()> {
+    fn update_metadata(&mut self, bp: &BundlePack) -> Result<()> {
         // TODO: check for duplicates, update, remove etc
-        if !self.bundles.contains_key(bp.id())? {
+        if !self.metadata.contains_key(bp.id())? {
             bail!("Bundle not in store!");
         }
-        self.bundles.insert(bp.id().to_string(), bp.to_cbor())?;
-        self.bundles.flush()?;
+        self.metadata.insert(bp.id().to_string(), bp.to_cbor())?;
+        self.metadata.flush()?;
         Ok(())
     }
-    fn remove(&mut self, bid: &str) -> Option<BundlePack> {
-        let res = self
-            .bundles
-            .remove(bid)
-            .map(|b| b.unwrap().as_ref().into())
-            .ok();
+    fn remove(&mut self, bid: &str) -> Result<()> {
+        let res = self.bundles.remove(bid)?;
         if let Err(err) = self.bundles.flush() {
             error!("Could not flush database: {}", err);
         }
 
-        res
+        Ok(())
     }
-    fn get(&self, bpid: &str) -> Option<BundlePack> {
+    /*fn get(&self, bpid: &str) -> Option<BundlePack> {
         self.bundles
             .get(bpid)
             .map(|b| b.unwrap().as_ref().into())
             .ok()
-    }
+    }*/
     fn count(&self) -> u64 {
         self.bundles.len() as u64
     }
@@ -69,7 +78,7 @@ impl BundleStore for SledBundleStore {
         }
     }
     fn pending(&self) -> Vec<String> {
-        self.bundles
+        self.metadata
             .iter()
             .values()
             .filter_map(Result::ok)
@@ -82,21 +91,9 @@ impl BundleStore for SledBundleStore {
             .map(|k| k.id().into())
             .collect()
     }
-    fn ready(&self) -> Vec<String> {
-        self.bundles
-            .iter()
-            .values()
-            .filter_map(Result::ok)
-            .map(|k| BundlePack::from(k.as_ref()))
-            .filter(|e| {
-                !e.has_constraint(Constraint::ReassemblyPending)
-                    && !e.has_constraint(Constraint::Contraindicated)
-            })
-            .map(|k| k.id().into())
-            .collect::<Vec<String>>()
-    }
+
     fn forwarding(&self) -> Vec<String> {
-        self.bundles
+        self.metadata
             .iter()
             .values()
             .filter_map(Result::ok)
@@ -106,13 +103,27 @@ impl BundleStore for SledBundleStore {
             .collect()
     }
     fn bundles(&self) -> Vec<BundlePack> {
-        self.bundles
+        self.metadata
             .iter()
             .values()
             .filter_map(Result::ok)
             .map(|k| BundlePack::from(k.as_ref()))
             .collect()
     }
+
+    fn get_bundle(&self, bpid: &str) -> Option<bp7::Bundle> {
+        self.bundles
+            .get(bpid)
+            .map(|b| Bundle::try_from(b.unwrap().as_ref().to_vec()).unwrap())
+            .ok()
+    }
+
+    fn get_metadata(&self, bpid: &str) -> Option<BundlePack> {
+        self.metadata
+            .get(bpid)
+            .map(|b| b.unwrap().as_ref().into())
+            .ok()
+    }
 }
 
 impl SledBundleStore {
@@ -122,6 +133,7 @@ impl SledBundleStore {
         let db = sled::open(wd).expect("open sled bundle store");
         SledBundleStore {
             bundles: db.open_tree("bundles").expect("cannot open bundles tree"),
+            metadata: db.open_tree("metadata").expect("cannot open bundles tree"),
         }
     }
 }
