@@ -1,8 +1,8 @@
 use crate::core::application_agent::ApplicationAgent;
 use crate::core::application_agent::SimpleApplicationAgent;
 use crate::core::helpers::rnd_peer;
+use crate::core::peer::PeerType;
 use crate::core::store::BundleStore;
-use crate::core::{bundlepack::BundlePack, peer::PeerType};
 use crate::dtnd::ws::WsAASession;
 use crate::peers_count;
 use crate::DtnConfig;
@@ -11,7 +11,6 @@ use crate::DTNCORE;
 use crate::PEERS;
 use crate::STATS;
 use crate::STORE;
-use actix::*;
 use actix_web::dev::RequestHead;
 use actix_web::HttpResponse;
 use actix_web::{
@@ -116,7 +115,7 @@ async fn index() -> impl Responder {
         janitor,
         timeout,
         num_peers: peers_count(),
-        num_bundles: (*DTNCORE.lock()).bundles().len(),
+        num_bundles: (*DTNCORE.lock()).bundle_count(),
     };
 
     let rendered = tt
@@ -208,7 +207,7 @@ async fn status_eids() -> String {
 }
 #[get("/status/bundles")]
 async fn status_bundles() -> String {
-    serde_json::to_string_pretty(&(*DTNCORE.lock()).bundles()).unwrap()
+    serde_json::to_string_pretty(&(*DTNCORE.lock()).bundle_ids()).unwrap()
 }
 #[get("/status/bundles_dest")]
 async fn status_bundles_dest() -> String {
@@ -378,8 +377,15 @@ async fn push_post(mut body: web::Payload) -> Result<String> {
     debug!("Received: {:?}", b_len);
     if let Ok(bndl) = bp7::Bundle::try_from(bytes.to_vec()) {
         info!("Received bundle {}", bndl.id());
-        crate::core::processing::receive(bndl.into()).await;
-        Ok(format!("Received {} bytes", b_len))
+        if let Err(err) = crate::core::processing::receive(bndl).await {
+            warn!("Error processing bundle: {}", err);
+            Err(actix_web::error::ErrorBadRequest(anyhow!(format!(
+                "Error processing bundle: {}",
+                err
+            ))))
+        } else {
+            Ok(format!("Received {} bytes", b_len))
+        }
     } else {
         Err(actix_web::error::ErrorBadRequest(anyhow!(
             "Error decoding bundle!"
@@ -534,8 +540,8 @@ async fn endpoint_hex(req: HttpRequest) -> Result<String> {
 #[get("/download")]
 async fn download(req: HttpRequest) -> Result<HttpResponse> {
     let bid = req.query_string();
-    if let Some(bundlepack) = (*STORE.lock()).get(&bid) {
-        let cbor_bundle = bundlepack.bundle.clone().to_cbor();
+    if let Some(bundle) = (*STORE.lock()).get_bundle(&bid) {
+        let cbor_bundle = bundle.clone().to_cbor();
         Ok(HttpResponse::Ok()
             .content_type("application/octet-stream")
             .body(cbor_bundle))
@@ -549,8 +555,8 @@ async fn download(req: HttpRequest) -> Result<HttpResponse> {
 #[get("/download.hex")]
 async fn download_hex(req: HttpRequest) -> Result<String> {
     let bid = req.query_string();
-    if let Some(bundlepack) = (*STORE.lock()).get(&bid) {
-        Ok(bp7::helpers::hexify(&bundlepack.bundle.clone().to_cbor()))
+    if let Some(bundle) = (*STORE.lock()).get_bundle(&bid) {
+        Ok(bp7::helpers::hexify(&bundle.clone().to_cbor()))
     } else {
         Err(actix_web::error::ErrorBadRequest(anyhow!(
             "Bundle not found"
