@@ -13,72 +13,56 @@ use std::net::SocketAddr;
 use tokio::net::UdpSocket;
 use tokio::time::interval;
 
-struct Server {
-    socket: UdpSocket,
-    buf: Vec<u8>,
-}
+async fn receiver(mut socket: UdpSocket) -> Result<(), io::Error> {
+    let mut buf: Vec<u8> = vec![0; 1024 * 64];
+    loop {
+        if let Ok((size, peer)) = socket.recv_from(&mut buf).await {
+            debug!("received {} bytes", size);
+            let deserialized: Beacon = match serde_cbor::from_slice(&buf[..size]) {
+                Ok(pkt) => pkt,
+                Err(e) => {
+                    error!("Deserialization of Beacon failed!{}", e);
+                    continue;
+                }
+            };
 
-impl Server {
-    async fn run(self) -> Result<(), io::Error> {
-        let Server {
-            mut socket,
-            mut buf,
-        } = self;
-
-        loop {
-            // Server received a Beacon
-            if let Some((size, peer)) = Some(socket.recv_from(&mut buf).await?) {
-                debug!("Beacon received");
-                let deserialized: Beacon = match serde_cbor::from_slice(&buf[..size]) {
-                    Ok(pkt) => pkt,
-                    Err(e) => {
-                        error!("Deserialization of Beacon failed!{}", e);
-                        continue;
-                    }
-                };
-                //let amt = try_ready!(self.socket.poll_send_to(&self.buf[..size], &peer));
-                //println!("Echoed {}/{} bytes to {}", amt, size, peer);
-                info!("Beacon from {}", peer);
-                debug!(":\n{}", deserialized);
-                // Creates a new peer from received beacon
-                let dtnpeer = DtnPeer::new(
-                    deserialized.eid().clone(),
-                    peer.ip(),
-                    PeerType::Dynamic,
-                    deserialized.beacon_period().clone(),
-                    deserialized.service_block().clas().clone(),
-                    deserialized.service_block().convert_services(),
-                );
-                peers_add(dtnpeer);
-                routing_notify(RoutingNotifcation::EncounteredPeer(&deserialized.eid()))
-            }
+            info!("Beacon from {} (len={})", peer, size);
+            debug!(":\n{}", deserialized);
+            // Creates a new peer from received beacon
+            let dtnpeer = DtnPeer::new(
+                deserialized.eid().clone(),
+                peer.ip(),
+                PeerType::Dynamic,
+                deserialized.beacon_period().clone(),
+                deserialized.service_block().clas().clone(),
+                deserialized.service_block().convert_services(),
+            );
+            peers_add(dtnpeer);
+            routing_notify(RoutingNotifcation::EncounteredPeer(&deserialized.eid()))
         }
     }
 }
 
-async fn announcer(socket: std::net::UdpSocket, _v6: bool) {
+async fn announcer(mut socket: UdpSocket, _v6: bool) {
     let mut task = interval(crate::CONFIG.lock().announcement_interval);
-    let mut sock = UdpSocket::from_std(socket).unwrap();
     loop {
+        debug!("waiting announcer");
         task.tick().await;
         debug!("running announcer");
 
         // Start to build beacon announcement
         let eid = (*CONFIG.lock()).host_eid.clone();
-
         let beacon_period = if !crate::CONFIG.lock().enable_period {
             None
         } else {
             Some(crate::CONFIG.lock().announcement_interval)
         };
         let mut pkt = Beacon::with_config(eid, ServiceBlock::new(), beacon_period);
-
         // Get all available clas
         &(*DTNCORE.lock())
             .cl_list
             .iter()
             .for_each(|cla| pkt.add_cla(&cla.name().to_string(), &Some(cla.port())));
-
         // Get all available services
         &(*DTNCORE.lock())
             .service_list
@@ -89,14 +73,12 @@ async fn announcer(socket: std::net::UdpSocket, _v6: bool) {
         //let addr = "127.0.0.1:3003".parse().unwrap();
 
         let mut destinations: HashMap<SocketAddr, u32> = HashMap::new();
-
         &(*CONFIG.lock())
             .discovery_destinations
             .iter()
             .for_each(|(key, value)| {
                 destinations.insert(key.clone().parse().unwrap(), *value);
             });
-
         for (destination, bsn) in destinations {
             &(*CONFIG.lock()).update_beacon_sequence_number(&destination.to_string());
             pkt.set_beacon_sequence_number(bsn);
@@ -112,12 +94,14 @@ async fn announcer(socket: std::net::UdpSocket, _v6: bool) {
                     pkt, destination
                 );
             }
-
-            if let Err(err) = sock
+            match socket
                 .send_to(&serde_cbor::to_vec(&pkt).unwrap(), destination)
                 .await
             {
-                error!("Sending announcement failed: {}", err);
+                Ok(amt) => {
+                    debug!("sent announcement (len={})", amt)
+                }
+                Err(err) => error!("Sending announcement failed: {}", err),
             }
         }
     }
@@ -152,20 +136,15 @@ pub async fn spawn_neighbour_discovery() -> Result<()> {
             .join_multicast_v4(&"224.0.0.26".parse()?, &std::net::Ipv4Addr::new(0, 0, 0, 0))
             .expect("error joining multicast v4 group");
         */
-        let socket_clone = socket.try_clone()?;
-        let sock = UdpSocket::from_std(socket)?;
 
-        info!("Listening on {}", sock.local_addr()?);
-        let server = Server {
-            socket: sock,
-            buf: vec![0; 1024],
-        };
-        tokio::spawn(server.run());
+        let socket1 = UdpSocket::from_std(socket.try_clone()?)?;
+        let socket2 = UdpSocket::from_std(socket.try_clone()?)?;
 
-        tokio::spawn(announcer(
-            socket_clone.try_clone().expect("couldn't clone the socket"),
-            false,
-        ));
+        info!("Listening on {}", socket1.local_addr()?);
+
+        tokio::spawn(receiver(socket1));
+
+        tokio::spawn(announcer(socket2, false));
     }
     if v6 {
         let addr: std::net::SocketAddr = format!("[::1]:{}", port).parse()?;
@@ -191,20 +170,14 @@ pub async fn spawn_neighbour_discovery() -> Result<()> {
             .join_multicast_v6(&"FF02::300".parse()?, 0)
             .expect("error joining multicast v6 group");
         */
-        let socket_clone = socket.try_clone()?;
-        let sock = UdpSocket::from_std(socket)?;
+        let socket1 = UdpSocket::from_std(socket.try_clone()?)?;
+        let socket2 = UdpSocket::from_std(socket.try_clone()?)?;
 
-        info!("Listening on {}", sock.local_addr()?);
-        let server = Server {
-            socket: sock,
-            buf: vec![0; 1024],
-        };
-        tokio::spawn(server.run());
+        info!("Listening on {}", socket1.local_addr()?);
 
-        tokio::spawn(announcer(
-            socket_clone.try_clone().expect("couldn't clone the socket"),
-            true,
-        ));
+        tokio::spawn(receiver(socket1));
+
+        tokio::spawn(announcer(socket2, true));
     }
 
     Ok(())
